@@ -189,6 +189,14 @@ sFractal::sFractal(const cParameterContainer *container)
 	surfFolds.orderOfFolds5 =
 		enumMulti_orderOfFolds(container->Get<int>("surfFolds_order_of_folds_5"));
 
+	// asurf3Folds
+	aSurf3Folds.orderOf3Folds1 =
+		enumMulti_orderOf3Folds(container->Get<int>("aSurf3Folds_order_of_folds_1"));
+	aSurf3Folds.orderOf3Folds2 =
+		enumMulti_orderOf3Folds(container->Get<int>("aSurf3Folds_order_of_folds_2"));
+	aSurf3Folds.orderOf3Folds3 =
+		enumMulti_orderOf3Folds(container->Get<int>("aSurf3Folds_order_of_folds_3"));
+
 	// benesi mag transforms
 	magTransf.orderOfTransf1 =
 		enumMulti_orderOfTransf(container->Get<int>("magTransf_order_of_transf_1"));
@@ -234,60 +242,26 @@ sFractal::sFractal(const cParameterContainer *container)
 	analyticDE.offset1 = container->Get<double>("analyticDE_offset_1");
 	analyticDE.offset2 = container->Get<double>("analyticDE_offset_2");
 	analyticDE.factor2 = container->Get<double>("analyticDE_factor_2");
+	analyticDE.enabled = container->Get<bool>("analyticDE_enabled");
 	analyticDE.enabledFalse = container->Get<bool>("analyticDE_enabled_false");
 	analyticDE.enabledAuxR2False = container->Get<bool>("analyticDE_enabled_auxR2_false");
 	analyticDE.scaleLin = container->Get<double>("analyticDE_scale_linear");
 	analyticDE.offsetLin = container->Get<double>("analyticDE_offset_linear");
 
-	// foldColor.factor000 = CVector4(container->Get<CVector3>("fold_color_factor_000"), 0.0);
-	// foldColor.factorMinR0 = container->Get<double>("fold_color_factorMinR0");
-	// foldColor.factorMaxR0 = container->Get<double>("fold_color_factorMaxR0");
-	foldColor.compMinR0 = container->Get<double>("fold_color_comp_minR0");
-	foldColor.compMinR = container->Get<double>("fold_color_comp_minR");
 	foldColor.auxColorEnabled = container->Get<bool>("fold_color_aux_color_enabled");
 	foldColor.auxColorEnabledFalse = container->Get<bool>("fold_color_aux_color_enabled_false");
 
-	foldColor.compFold = container->Get<double>("fold_color_comp_fold");
-	foldColor.compFold0 = container->Get<double>("fold_color_comp_fold0");
-
-	foldColor.radiusColorEnabledFalse = container->Get<bool>("fold_color_radius_color_enabled_false");
-	foldColor.compRadius0 = container->Get<double>("fold_color_comp_radius0");
-
-	foldColor.deColorEnabledFalse = container->Get<bool>("fold_color_de_color_enabled_false");
-	foldColor.deScale0 = container->Get<double>("fold_color_de_scale0");
-
-	// foldColor.oldHybridEnabledFalse = container->Get<bool>("fold_color_old_hybrid_enabled_false");
-	foldColor.oldScale0 = container->Get<double>("fold_color_old_scale0");
-	// foldColor.oldScale1 = container->Get<double>("fold_color_old_scale1");
-	// foldColor.xyzColorEnabledFalse = container->Get<bool>("fold_color_xyz_color_enabled_false");
-	// foldColor.xyz000 = CVector4(container->Get<CVector3>("fold_color_xyz_000"), 0.0);
-	// foldColor.temp1EnabledFalse = container->Get<bool>("fold_color_temp1_enabled_false");
-	// foldColor.extraModeEnabledFalse = container->Get<bool>("fold_color_extra_mode_enabled_false");
-	foldColor.compScale = container->Get<double>("fold_color_comp_scale");
-	foldColor.colorMin = container->Get<double>("fold_color_color_min");
-
-	// foldColor.newScale0 = container->Get<double>("fold_color_new_scale0");
-	// foldColor.parabScale0 = container->Get<double>("fold_color_parab_scale0");
 	foldColor.scaleA0 = container->Get<double>("fold_color_scaleA0");
 	foldColor.scaleB0 = container->Get<double>("fold_color_scaleB0");
-	// foldColor.scaleC0 = container->Get<double>("fold_color_scaleC0");
 	foldColor.scaleD0 = container->Get<double>("fold_color_scaleD0");
-	foldColor.scaleE0 = container->Get<double>("fold_color_scaleE0");
 	foldColor.scaleF0 = container->Get<double>("fold_color_scaleF0");
-	// foldColor.scaleG0 = container->Get<double>("fold_color_scaleG0");
 	foldColor.scaleA1 = container->Get<double>("fold_color_scaleA1");
 	foldColor.scaleB1 = container->Get<double>("fold_color_scaleB1");
-	// foldColor.period1 = container->Get<double>("fold_color_period1");
-	// foldColor.trigAdd1 = container->Get<double>("fold_color_trig_add1");
-
-	foldColor.limitMin0 = container->Get<double>("fold_color_limit_min0");
-	foldColor.limitMax9999 = container->Get<double>("fold_color_limit_max9999");
 
 	foldColor.intAx0 = container->Get<int>("fold_color_int_Ax0");
 	foldColor.intAy0 = container->Get<int>("fold_color_int_Ay0");
 	foldColor.intAz0 = container->Get<int>("fold_color_int_Az0");
-	// foldColor.parabEnabledFalse = container->Get<bool>("fold_color_parab_enabled_false");
-	// foldColor.cosEnabledFalse = container->Get<bool>("fold_color_cos_enabled_false");
+
 	foldColor.distanceEnabledFalse = container->Get<bool>("fold_color_distance_enabled_false");
 
 	// common parameters for transforming formulas
@@ -345,6 +319,7 @@ sFractal::sFractal(const cParameterContainer *container)
 	transformCommon.intA = container->Get<int>("transf_int_A");
 	transformCommon.intB = container->Get<int>("transf_int_B");
 	transformCommon.int1 = container->Get<int>("transf_int_1");
+	transformCommon.int6 = container->Get<int>("transf_int_6");
 	transformCommon.int8X = container->Get<int>("transf_int8_X");
 	transformCommon.int8Y = container->Get<int>("transf_int8_Y");
 	transformCommon.int8Z = container->Get<int>("transf_int8_Z");
@@ -458,6 +433,8 @@ sFractal::sFractal(const cParameterContainer *container)
 
 	transformCommon.rotation = container->Get<CVector3>("transf_rotation");
 	transformCommon.rotation2 = container->Get<CVector3>("transf_rotation2");
+	transformCommon.rotationVary = container->Get<CVector3>("transf_rotationVary");
+
 	transformCommon.rotation44a =
 		container->Get<CVector3>("transf_rotation44a"); //...........................
 	transformCommon.rotation44b =
@@ -532,6 +509,7 @@ sFractal::sFractal(const cParameterContainer *container)
 	transformCommon.functionEnabledXFalse = container->Get<bool>("transf_function_enabledX_false");
 	transformCommon.juliaMode = container->Get<bool>("transf_constant_julia_mode");
 	transformCommon.rotationEnabled = container->Get<bool>("transf_rotation_enabled");
+	transformCommon.rotation2EnabledFalse = container->Get<bool>("transf_rotation2_enabled_false");
 
 	WriteLog("cFractal::RecalculateFractalParams(void)", 2);
 
@@ -547,7 +525,7 @@ void sFractal::RecalculateFractalParams()
 		IFS.direction[i].Normalize();
 	}
 
-	mandelbox.mainRot.SetRotation2(mandelbox.rotationMain * (M_PI / 180.0));
+	mandelbox.mainRot.SetRotation2(mandelbox.rotationMain * M_PI_180);
 
 	for (int fold = 0; fold < MANDELBOX_FOLDS; ++fold)
 	{
@@ -572,7 +550,7 @@ void sFractal::RecalculateFractalParams()
 		transformCommon.rotation44b * (M_PI / 180.0)); // ...............................
 	transformCommon.rotationMatrix.SetRotation2(transformCommon.rotation * (M_PI / 180.0));
 	transformCommon.rotationMatrix2.SetRotation2(transformCommon.rotation2 * (M_PI / 180.0));
-	transformCommon.tempRotMatrix.SetRotation2(transformCommon.rotation * (M_PI / 180.0));
+	transformCommon.rotationMatrixVary.SetRotation2(transformCommon.rotationVary * M_PI_180);
 
 	transformCommon.sqtR = sqrt(transformCommon.minR05);
 	transformCommon.mboxFactor1 = 1.0 / transformCommon.sqtR; // hmmm??
