@@ -82,6 +82,7 @@ void AboxMod12Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux)
 void AboxMod13Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void AboxModKaliIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void AboxModKaliEiffieIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
+void AboxModKaliV2Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void AboxVSIcen1Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void AexionIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void AexionOctopusModIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
@@ -119,8 +120,8 @@ void LkmitchIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void Makin3d2Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 
 void MandelboxIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
-
 void MandelboxMengerIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
+void MandelboxVariableIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void Mandelbulb2Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void Mandelbulb3Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void Mandelbulb4Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
@@ -216,6 +217,8 @@ void TransfMultipleAngleIteration(CVector4 &z, const sFractal *fractal, sExtende
 void TransfOctoFoldIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void TransfParabFoldIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void TransfPlatonicSolidIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
+void TransfPolyFoldSymIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
+void TransfPolyFoldSymMultiIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void TransfRPowerIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void TransfPwr2PolynomialIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void TransfQuaternionFoldIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
@@ -251,6 +254,7 @@ void TransfZvectorAxisSwapIteration(CVector4 &z, const sFractal *fractal, sExten
 // 4D
 void Abox4dIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void Bristorbrot4dIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
+void BoxFold4dBulbPow2Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void MandelboxVaryScale4dIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void MixPinski4dIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void Menger4dIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
