@@ -118,8 +118,10 @@ QStringList cMaterial::paramsList = {
 
 	"fractal_coloring_rad_enabled_false", "fractal_coloring_rad_weight",
 	"fractal_coloring_rad_squared_enabled_false", "fractal_coloring_rad_div_1e13_false",
+
 	"fractal_coloring_rad_div_de_enabled_false", "fractal_coloring_rad_div_log_de_false",
 	"fractal_coloring_rad_div_de_weight", "fractal_coloring_rad_div_de_squared_false",
+	"fractal_coloring_rad_div_de_1e13_false",
 
 	"fractal_coloring_xyz_bias_enabled_false", "fractal_coloring_xyz_000",
 	"fractal_coloring_xyz_iter_scale", "fractal_coloring_xyz_x_sqrd_enabled_false",
@@ -146,10 +148,12 @@ QStringList cMaterial::paramsList = {
 	"fractal_coloring_max_color_value", "fractal_coloring_min_color_value",
 };
 
-void cMaterial::setParameters(int _id, const cParameterContainer *materialParam, bool quiet = false)
+void cMaterial::setParameters(int _id, const cParameterContainer *materialParam, bool quiet)
 {
 	id = _id;
 
+	int frameNo = materialParam->Get<int>("frame_no");
+
 	shading = materialParam->Get<double>(Name("shading", id));
 	specular = materialParam->Get<double>(Name("specular", id));
 	specularWidth = materialParam->Get<double>(Name("specular_width", id));
@@ -241,7 +245,6 @@ void cMaterial::setParameters(int _id, const cParameterContainer *materialParam,
 		materialParam->Get<double>(Name("fractal_coloring_orbit_trap_weight", id));
 	fractalColoring.initialMinimumR =
 		materialParam->Get<double>(Name("fractal_coloring_initial_minimumR", id));
-
 	// aux.color
 	fractalColoring.auxColorFalse =
 		materialParam->Get<double>(Name("fractal_coloring_aux_color_false", id));
@@ -249,8 +252,7 @@ void cMaterial::setParameters(int _id, const cParameterContainer *materialParam,
 		materialParam->Get<double>(Name("fractal_coloring_aux_color_weight", id));
 	fractalColoring.auxColorHybridWeight =
 		materialParam->Get<double>(Name("fractal_coloring_aux_color_hybrid_weight", id));
-
-	// radius functions
+	// radius
 	fractalColoring.radFalse =
 		materialParam->Get<bool>(Name("fractal_coloring_rad_enabled_false", id));
 	fractalColoring.radWeight = materialParam->Get<double>(Name("fractal_coloring_rad_weight", id));
@@ -258,7 +260,7 @@ void cMaterial::setParameters(int _id, const cParameterContainer *materialParam,
 		materialParam->Get<bool>(Name("fractal_coloring_rad_squared_enabled_false", id));
 	fractalColoring.radDiv1e13False =
 		materialParam->Get<bool>(Name("fractal_coloring_rad_div_1e13_false", id));
-
+	// radius/DE
 	fractalColoring.radDivDeFalse =
 		materialParam->Get<bool>(Name("fractal_coloring_rad_div_de_enabled_false", id));
 	fractalColoring.radDivLogDeFalse =
@@ -267,7 +269,9 @@ void cMaterial::setParameters(int _id, const cParameterContainer *materialParam,
 		materialParam->Get<double>(Name("fractal_coloring_rad_div_de_weight", id));
 	fractalColoring.radDivDeSquaredFalse =
 		materialParam->Get<bool>(Name("fractal_coloring_rad_div_de_squared_false", id));
-
+	fractalColoring.radDivDE1e13False =
+		materialParam->Get<bool>(Name("fractal_coloring_rad_div_de_1e13_false", id));
+	// XYZ bias
 	fractalColoring.xyzBiasEnabledFalse =
 		materialParam->Get<bool>(Name("fractal_coloring_xyz_bias_enabled_false", id));
 	fractalColoring.xyz000 = materialParam->Get<CVector3>(Name("fractal_coloring_xyz_000", id));
@@ -332,28 +336,33 @@ void cMaterial::setParameters(int _id, const cParameterContainer *materialParam,
 	{
 		if (useColorTexture)
 			colorTexture.FromQByteArray(
-				gNetRender->GetTexture(materialParam->Get<QString>(Name("file_color_texture", id))),
+				gNetRender->GetTexture(
+					materialParam->Get<QString>(Name("file_color_texture", id)), frameNo),
 				cTexture::useMipmaps);
 
 		if (useDiffusionTexture)
 			diffusionTexture.FromQByteArray(
-				gNetRender->GetTexture(materialParam->Get<QString>(Name("file_diffusion_texture", id))),
+				gNetRender->GetTexture(
+					materialParam->Get<QString>(Name("file_diffusion_texture", id)), frameNo),
 				cTexture::useMipmaps);
 
 		if (useLuminosityTexture)
 			luminosityTexture.FromQByteArray(
-				gNetRender->GetTexture(materialParam->Get<QString>(Name("file_luminosity_texture", id))),
+				gNetRender->GetTexture(
+					materialParam->Get<QString>(Name("file_luminosity_texture", id)), frameNo),
 				cTexture::useMipmaps);
 
 		if (useDisplacementTexture)
 			displacementTexture.FromQByteArray(
-				gNetRender->GetTexture(materialParam->Get<QString>(Name("file_displacement_texture", id))),
+				gNetRender->GetTexture(
+					materialParam->Get<QString>(Name("file_displacement_texture", id)), frameNo),
 				cTexture::doNotUseMipmaps);
 
 		if (useNormalMapTexture)
 		{
 			normalMapTexture.FromQByteArray(
-				gNetRender->GetTexture(materialParam->Get<QString>(Name("file_normal_map_texture", id))),
+				gNetRender->GetTexture(
+					materialParam->Get<QString>(Name("file_normal_map_texture", id)), frameNo),
 				cTexture::doNotUseMipmaps);
 
 			if (materialParam->Get<bool>(Name("normal_map_texture_invert_green", id)))
@@ -363,26 +372,26 @@ void cMaterial::setParameters(int _id, const cParameterContainer *materialParam,
 	else
 	{
 		if (useColorTexture)
-			colorTexture = cTexture(
-				materialParam->Get<QString>(Name("file_color_texture", id)), cTexture::useMipmaps, quiet);
+			colorTexture = cTexture(materialParam->Get<QString>(Name("file_color_texture", id)),
+				cTexture::useMipmaps, frameNo, quiet);
 
 		if (useDiffusionTexture)
 			diffusionTexture = cTexture(materialParam->Get<QString>(Name("file_diffusion_texture", id)),
-				cTexture::useMipmaps, quiet);
+				cTexture::useMipmaps, frameNo, quiet);
 
 		if (useLuminosityTexture)
 			luminosityTexture = cTexture(materialParam->Get<QString>(Name("file_luminosity_texture", id)),
-				cTexture::useMipmaps, quiet);
+				cTexture::useMipmaps, frameNo, quiet);
 
 		if (useDisplacementTexture)
 			displacementTexture =
 				cTexture(materialParam->Get<QString>(Name("file_displacement_texture", id)),
-					cTexture::doNotUseMipmaps, quiet);
+					cTexture::doNotUseMipmaps, frameNo, quiet);
 
 		if (useNormalMapTexture)
 		{
 			normalMapTexture = cTexture(materialParam->Get<QString>(Name("file_normal_map_texture", id)),
-				cTexture::useMipmaps, quiet);
+				cTexture::useMipmaps, frameNo, quiet);
 			if (materialParam->Get<bool>(Name("normal_map_texture_invert_green", id)))
 				normalMapTexture.SetInvertGreen(true);
 		}
@@ -394,6 +403,7 @@ void cMaterial::setParameters(int _id, const cParameterContainer *materialParam,
 void CreateMaterialsMap(
 	const cParameterContainer *params, QMap<int, cMaterial> *materials, bool quiet)
 {
+	materials->clear();
 	QList<QString> listOfParameters = params->GetListOfParameters();
 	for (int i = 0; i < listOfParameters.size(); i++)
 	{
