@@ -57,6 +57,7 @@
 #include "rectangle.hpp"
 #include "render_data.hpp"
 #include "render_worker.hpp"
+#include "texture.hpp"
 
 // custom includes
 #ifdef USE_OPENCL
@@ -70,6 +71,8 @@ cOpenClEngineRenderFractal::cOpenClEngineRenderFractal(cOpenClHardware *_hardwar
 #ifdef USE_OPENCL
 	constantInBuffer = nullptr;
 	inCLConstBuffer = nullptr;
+	backgroungImageBuffer = nullptr;
+	backgroundImage2D = nullptr;
 
 	inCLBuffer = nullptr;
 
@@ -79,6 +82,8 @@ cOpenClEngineRenderFractal::cOpenClEngineRenderFractal(cOpenClHardware *_hardwar
 	autoRefreshMode = false;
 	monteCarlo = false;
 
+	renderEngineMode = clRenderEngineTypeNone;
+
 #endif
 }
 
@@ -99,6 +104,9 @@ void cOpenClEngineRenderFractal::ReleaseMemory()
 	inCLConstBuffer = nullptr;
 	if (inCLBuffer) delete inCLBuffer;
 	inCLBuffer = nullptr;
+	if (backgroungImageBuffer) delete backgroungImageBuffer;
+	backgroungImageBuffer = nullptr;
+
 	dynamicData->Clear();
 }
 
@@ -131,7 +139,11 @@ bool cOpenClEngineRenderFractal::LoadSourcesAndCompile(const cParameterContainer
 		clHeaderFiles.append("fractparams_cl.hpp");
 		clHeaderFiles.append("fractal_sequence_cl.h");
 		clHeaderFiles.append("material_cl.h");
+		clHeaderFiles.append("object_type_cl.h");
+		clHeaderFiles.append("object_data_cl.h");
+		clHeaderFiles.append("primitives_cl.h");
 		clHeaderFiles.append("input_data_structures.h");
+		clHeaderFiles.append("render_data_cl.h");
 
 		// pass through define constants
 		programEngine.append("#define USE_OPENCL 1\n");
@@ -166,7 +178,7 @@ bool cOpenClEngineRenderFractal::LoadSourcesAndCompile(const cParameterContainer
 			}
 		}
 
-		if (params->Get<int>("opencl_mode") != clRenderEngineTypeFast)
+		if (renderEngineMode != clRenderEngineTypeFast)
 		{
 			// fractal coloring
 			programEngine.append("#include \"" + openclEnginePath + "fractal_coloring.cl\"\n");
@@ -175,6 +187,9 @@ bool cOpenClEngineRenderFractal::LoadSourcesAndCompile(const cParameterContainer
 		// compute fractal
 		programEngine.append("#include \"" + openclEnginePath + "compute_fractal.cl\"\n");
 
+		// compute fractal
+		programEngine.append("#include \"" + openclEnginePath + "primitives.cl\"\n");
+
 		// calculate distance
 		programEngine.append("#include \"" + openclEnginePath + "calculate_distance.cl\"\n");
 
@@ -184,19 +199,19 @@ bool cOpenClEngineRenderFractal::LoadSourcesAndCompile(const cParameterContainer
 		// 3D projections (3point, equirectagular, fisheye)
 		programEngine.append("#include \"" + openclEnginePath + "projection_3d.cl\"\n");
 
-		if (params->Get<int>("opencl_mode") != clRenderEngineTypeFast)
+		if (renderEngineMode != clRenderEngineTypeFast)
 		{
 			// shaders
 			programEngine.append("#include \"" + openclEnginePath + "shaders.cl\"\n");
 		}
 
-		if (params->Get<int>("opencl_mode") == clRenderEngineTypeFull)
+		if (renderEngineMode == clRenderEngineTypeFull)
 		{
 			// ray recursion
 			programEngine.append("#include \"" + openclEnginePath + "ray_recursion.cl\"\n");
 		}
 
-		if (params->Get<int>("opencl_mode") != clRenderEngineTypeFast)
+		if (renderEngineMode != clRenderEngineTypeFast)
 		{
 			// Monte Carlo DOF
 			programEngine.append("#include \"" + openclEnginePath + "monte_carlo_dof.cl\"\n");
@@ -204,7 +219,7 @@ bool cOpenClEngineRenderFractal::LoadSourcesAndCompile(const cParameterContainer
 
 		// main engine
 		QString engineFileName;
-		switch (enumClRenderEngineMode(params->Get<int>("opencl_mode")))
+		switch (renderEngineMode)
 		{
 			case clRenderEngineTypeFast: engineFileName = "fast_engine.cl"; break;
 			case clRenderEngineTypeLimited: engineFileName = "limited_engine.cl"; break;
@@ -221,6 +236,7 @@ bool cOpenClEngineRenderFractal::LoadSourcesAndCompile(const cParameterContainer
 	}
 
 	SetUseBuildCache(!params->Get<bool>("opencl_disable_build_cache"));
+	SetUseFastRelaxedMath(params->Get<bool>("opencl_use_fast_relaxed_math"));
 
 	// building OpenCl kernel
 	QString errorString;
@@ -253,22 +269,32 @@ void cOpenClEngineRenderFractal::SetParameters(const cParameterContainer *paramC
 
 	definesCollector.clear();
 
+	renderEngineMode = enumClRenderEngineMode(paramContainer->Get<int>("opencl_mode"));
+
 	// update camera rotation data (needed for simplified calculation in opencl kernel)
 	cCameraTarget cameraTarget(paramRender->camera, paramRender->target, paramRender->topVector);
 	paramRender->viewAngle = cameraTarget.GetRotation() * 180.0 / M_PI;
 	paramRender->resolution = 1.0 / paramRender->imageHeight;
 
+	if (renderEngineMode == clRenderEngineTypeFull) definesCollector += " -DFULL_ENGINE";
+
 	// define distance estimation method
 	fractal::enumDEType deType = fractals->GetDEType(0);
 	fractal::enumDEFunctionType deFunctionType = fractals->GetDEFunctionType(0);
 
 	if (fractals->IsHybrid()) definesCollector += " -DIS_HYBRID";
 
-	if (deType == fractal::analyticDEType)
+	bool useAnalyticDEType = false;
+	bool useDeltaDEType = false;
+	bool useLinearDEFunction = false;
+	bool useLogarithmicDEFunction = false;
+	bool usePseudoKleinianDEFunction = false;
+
+	if (fractals->IsHybrid())
 	{
-		definesCollector += " -DANALYTIC_DE";
-		if (fractals->IsHybrid())
+		if (deType == fractal::analyticDEType)
 		{
+			useAnalyticDEType = true;
 			switch (deFunctionType)
 			{
 				case fractal::linearDEFunction: definesCollector += " -DANALYTIC_LINEAR_DE"; break;
@@ -282,20 +308,47 @@ void cOpenClEngineRenderFractal::SetParameters(const cParameterContainer *paramC
 				default: break;
 			}
 		}
-	}
-	else if (deType == fractal::deltaDEType)
-	{
-		definesCollector += " -DDELTA_DE";
-		switch (deFunctionType)
+		if (deType == fractal::deltaDEType)
 		{
-			case fractal::linearDEFunction: definesCollector += " -DDELTA_LINEAR_DE"; break;
-			case fractal::logarithmicDEFunction: definesCollector += " -DDELTA_LOG_DE"; break;
-			case fractal::pseudoKleinianDEFunction:
-				definesCollector += " -DDELTA_PSEUDO_KLEINIAN_DE";
-				break;
-			default: break;
+			useDeltaDEType = true;
+			switch (fractals->GetDEFunctionType(0))
+			{
+				case fractal::linearDEFunction: useLinearDEFunction = true; break;
+				case fractal::logarithmicDEFunction: useLogarithmicDEFunction = true; break;
+				case fractal::pseudoKleinianDEFunction: usePseudoKleinianDEFunction = true; break;
+				default: break;
+			}
 		}
 	}
+	else // is not Hybrid
+	{
+		for (int i = 0; i < NUMBER_OF_FRACTALS; i++)
+		{
+			if (!paramRender->booleanOperatorsEnabled && i == 1) break;
+
+			if (fractals->GetDEType(i) == fractal::analyticDEType)
+			{
+				useAnalyticDEType = true;
+			}
+			else if (fractals->GetDEType(i) == fractal::deltaDEType)
+			{
+				useDeltaDEType = true;
+				switch (fractals->GetDEFunctionType(i))
+				{
+					case fractal::linearDEFunction: useLinearDEFunction = true; break;
+					case fractal::logarithmicDEFunction: useLogarithmicDEFunction = true; break;
+					case fractal::pseudoKleinianDEFunction: usePseudoKleinianDEFunction = true; break;
+					default: break;
+				}
+			}
+		}
+	}
+
+	if (useAnalyticDEType) definesCollector += " -DANALYTIC_DE";
+	if (useDeltaDEType) definesCollector += " -DDELTA_DE";
+	if (useLinearDEFunction) definesCollector += " -DDELTA_LINEAR_DE";
+	if (useLogarithmicDEFunction) definesCollector += " -DDELTA_LOG_DE";
+	if (usePseudoKleinianDEFunction) definesCollector += " -DDELTA_PSEUDO_KLEINIAN_DE";
 
 	if (paramRender->limitsEnabled) definesCollector += " -DLIMITS_ENABLED";
 
@@ -367,6 +420,8 @@ void cOpenClEngineRenderFractal::SetParameters(const cParameterContainer *paramC
 
 	if (paramRender->interiorMode) definesCollector += " -DINTERIOR_MODE";
 
+	if (paramRender->booleanOperatorsEnabled) definesCollector += " -DBOOLEAN_OPERATORS";
+
 	bool isVolumetricLight = false;
 	for (int i = 0; i < 5; i++)
 	{
@@ -404,6 +459,19 @@ void cOpenClEngineRenderFractal::SetParameters(const cParameterContainer *paramC
 	if (paramRender->DOFMonteCarlo && paramRender->DOFEnabled)
 		definesCollector += " -DMONTE_CARLO_DOF";
 
+	if (paramRender->texturedBackground)
+	{
+		definesCollector += " -DTEXTURED_BACKGROUND";
+		switch (paramRender->texturedBackgroundMapType)
+		{
+			case params::mapEquirectangular: definesCollector += " -DBACKGROUND_EQUIRECTANGULAR"; break;
+			case params::mapDoubleHemisphere:
+				definesCollector += " -DBACKGROUND_DOUBLE_HEMISPHERE";
+				break;
+			case params::mapFlat: definesCollector += " -DBACKGROUND_FLAT"; break;
+		}
+	}
+
 	listOfUsedFormulas = listOfUsedFormulas.toSet().toList(); // eliminate duplicates
 
 	WriteLogDouble("Constant buffer size [KB]", sizeof(sClInConstants) / 1024.0, 3);
@@ -415,7 +483,8 @@ void cOpenClEngineRenderFractal::SetParameters(const cParameterContainer *paramC
 	// materials
 	QMap<int, cMaterial> materials;
 	CreateMaterialsMap(paramContainer, &materials, true);
-	dynamicData->BuildMaterialsData(materials);
+	int materialsArraySize = dynamicData->BuildMaterialsData(materials);
+	definesCollector += " -DMAT_ARRAY_SIZE=" + QString::number(materialsArraySize);
 
 	bool anyMaterialIsReflective = false;
 	bool anyMaterialIsRefractive = false;
@@ -437,8 +506,7 @@ void cOpenClEngineRenderFractal::SetParameters(const cParameterContainer *paramC
 	if (anyMaterialIsRefractive) definesCollector += " -DUSE_REFRACTION";
 	if (anyMaterialHasSpecialColoring) definesCollector += " -DUSE_COLORING_MODES";
 	if (anyMaterialHasIridescence) definesCollector += " -DUSE_IRIDESCENCE";
-	if (paramContainer->Get<int>("opencl_mode") != clRenderEngineTypeFast
-			&& anyMaterialHasColoringEnabled)
+	if (renderEngineMode != clRenderEngineTypeFast && anyMaterialHasColoringEnabled)
 		definesCollector += " -DUSE_FRACTAL_COLORING";
 	if (anyMaterialHasExtraColoringEnabled) definesCollector += " -DUSE_EXTRA_COLORING";
 
@@ -461,6 +529,13 @@ void cOpenClEngineRenderFractal::SetParameters(const cParameterContainer *paramC
 	dynamicData->BuildAOVectorsData(AOVectors, numberOfVectors);
 	dynamicData->BuildLightsData(&renderData->lights);
 
+	dynamicData->BuildPrimitivesData(&paramRender->primitives);
+	if (paramRender->primitives.GetListOfPrimitives()->size() > 0)
+		definesCollector += " -DUSE_PRIMITIVES";
+
+	dynamicData->BuildObjectsData(&renderData->objectData);
+	// definesCollector += " -DOBJ_ARRAY_SIZE=" + QString::number(renderData->objectData.size());
+
 	dynamicData->FillHeader();
 
 	inBuffer = dynamicData->GetData();
@@ -468,7 +543,7 @@ void cOpenClEngineRenderFractal::SetParameters(const cParameterContainer *paramC
 	//---------------- another parameters -------------
 	autoRefreshMode = paramContainer->Get<bool>("auto_refresh");
 	monteCarlo = paramRender->DOFMonteCarlo && paramRender->DOFEnabled
-							 && paramContainer->Get<int>("opencl_mode") != clRenderEngineTypeFast;
+							 && renderEngineMode != clRenderEngineTypeFast;
 
 	// copy all cl parameters to constant buffer
 	constantInBuffer->params = clCopySParamRenderCl(*paramRender);
@@ -561,6 +636,10 @@ bool cOpenClEngineRenderFractal::Render(cImage *image, bool *stopRequest, sRende
 		QElapsedTimer progressRefreshTimer;
 		progressRefreshTimer.start();
 
+		QElapsedTimer timerImageRefresh;
+		timerImageRefresh.start();
+		int lastRefreshTime = 1;
+
 		qint64 numberOfPixels = qint64(width) * qint64(height);
 		qint64 gridWidth = width / optimalJob.stepSizeX;
 		qint64 gridHeight = height / optimalJob.stepSizeY;
@@ -578,9 +657,19 @@ bool cOpenClEngineRenderFractal::Render(cImage *image, bool *stopRequest, sRende
 			numberOfSamples = constantInBuffer->params.DOFSamples;
 		}
 
+		QList<QRect> lastRenderedRects;
+
 		double doneMC = 0.0f;
 		QList<QPoint> tileSequence = calculateOptimalTileSequence(gridWidth + 1, gridHeight + 1);
 
+		// writing data to queue
+		if (!WriteBuffersToQueue()) throw;
+
+		if (renderEngineMode != clRenderEngineTypeFast)
+		{
+			if (!PrepareBufferForBackground(renderData)) throw;
+		}
+
 		try
 		{
 
@@ -591,7 +680,7 @@ bool cOpenClEngineRenderFractal::Render(cImage *image, bool *stopRequest, sRende
 				// requires initialization for all opencl devices
 				// requires optimalJob for all opencl devices
 
-				QList<QRect> lastRenderedRects;
+				lastRenderedRects.clear();
 
 				qint64 pixelsRendered = 0;
 				qint64 pixelsRenderedMC = 0;
@@ -630,23 +719,23 @@ bool cOpenClEngineRenderFractal::Render(cImage *image, bool *stopRequest, sRende
 							// assign parameters to kernel
 							if (!AssignParametersToKernel()) throw;
 
-							// writing data to queue
-							if (!WriteBuffersToQueue()) throw;
-
-							if (!autoRefreshMode && !monteCarlo)
-							{
-								const QRect currentCorners = SizedRectangle(jobX, jobY, jobWidth2, jobHeight2);
-								MarkCurrentPendingTile(image, currentCorners);
-							}
+							//							if (!autoRefreshMode && !monteCarlo && progressRefreshTimer.elapsed()
+							//> 1000)
+							//							{
+							//								const QRect currentCorners = SizedRectangle(jobX, jobY, jobWidth2,
+							// jobHeight2);
+							//								MarkCurrentPendingTile(image, currentCorners);
+							//							}
 
 							// processing queue
 							if (!ProcessQueue(jobX, jobY, pixelsLeftX, pixelsLeftY)) throw;
 
 							// update image when OpenCl kernel is working
-							if (lastRenderedRects.size() > 0)
+							if (lastRenderedRects.size() > 0
+									&& timerImageRefresh.nsecsElapsed() > lastRefreshTime * 1000)
 							{
-								QElapsedTimer timerImageRefresh;
-								timerImageRefresh.start();
+								timerImageRefresh.restart();
+
 								image->NullPostEffect(&lastRenderedRects);
 								image->CompileImage(&lastRenderedRects);
 								if (image->IsPreview())
@@ -655,10 +744,10 @@ bool cOpenClEngineRenderFractal::Render(cImage *image, bool *stopRequest, sRende
 									image->UpdatePreview(&lastRenderedRects);
 									emit updateImage();
 								}
+								lastRefreshTime = timerImageRefresh.nsecsElapsed() / lastRenderedRects.size();
+
 								lastRenderedRects.clear();
-								optimalJob.optimalProcessingCycle = 2.0 * timerImageRefresh.elapsed() / 1000.0;
-								if (optimalJob.optimalProcessingCycle < 0.1)
-									optimalJob.optimalProcessingCycle = 0.1;
+								timerImageRefresh.restart();
 							}
 
 							if (!ReadBuffersFromQueue()) throw;
@@ -762,6 +851,16 @@ bool cOpenClEngineRenderFractal::Render(cImage *image, bool *stopRequest, sRende
 
 						if (*stopRequest)
 						{
+							image->NullPostEffect(&lastRenderedRects);
+							image->CompileImage(&lastRenderedRects);
+							if (image->IsPreview())
+							{
+								image->ConvertTo8bit(&lastRenderedRects);
+								image->UpdatePreview(&lastRenderedRects);
+								emit updateImage();
+							}
+							lastRenderedRects.clear();
+
 							delete[] noiseTable;
 							return false;
 						}
@@ -820,19 +919,19 @@ bool cOpenClEngineRenderFractal::Render(cImage *image, bool *stopRequest, sRende
 			"cOpenClEngineRenderFractal: OpenCL Rendering time [s]", timer.nsecsElapsed() / 1.0e9, 2);
 
 		// refresh image at end
-		image->NullPostEffect();
+		image->NullPostEffect(&lastRenderedRects);
 
 		WriteLog("image->CompileImage()", 2);
-		image->CompileImage();
+		image->CompileImage(&lastRenderedRects);
 
 		if (image->IsPreview())
 		{
 			WriteLog("image->ConvertTo8bit()", 2);
-			image->ConvertTo8bit();
-			WriteLog("image->UpdatePreview()", 2);
-			image->UpdatePreview();
-			WriteLog("image->GetImageWidget()->update()", 2);
-			emit updateImage();
+			image->ConvertTo8bit(&lastRenderedRects);
+			//			WriteLog("image->UpdatePreview()", 2);
+			//			image->UpdatePreview(&lastRenderedRects);
+			//			WriteLog("image->GetImageWidget()->update()", 2);
+			//			emit updateImage();
 		}
 
 		emit updateProgressAndStatus(tr("OpenCl - rendering finished"), progressText.getText(1.0), 1.0);
@@ -855,7 +954,7 @@ QList<QPoint> cOpenClEngineRenderFractal::calculateOptimalTileSequence(
 	}
 	qSort(tiles.begin(), tiles.end(),
 		std::bind(cOpenClEngineRenderFractal::sortByCenterDistanceAsc, std::placeholders::_1,
-					std::placeholders::_2, gridWidth, gridHeight));
+			std::placeholders::_2, gridWidth, gridHeight));
 	return tiles;
 }
 
@@ -957,8 +1056,21 @@ bool cOpenClEngineRenderFractal::AssignParametersToKernelAdditional(int argItera
 		return false;
 	}
 
+	if (renderEngineMode != clRenderEngineTypeFast)
+	{
+		err = kernel->setArg(
+			argIterator++, *backgroundImage2D); // input data in constant memory (faster than global)
+		if (!checkErr(err, "kernel->setArg(3, *backgroundImage2D)"))
+		{
+			emit showErrorMessage(
+				QObject::tr("Cannot set OpenCL argument for %2").arg(QObject::tr("background image")),
+				cErrorMessage::errorMessage, nullptr);
+			return false;
+		}
+	}
+
 	err = kernel->setArg(argIterator++, Random(1000000)); // random seed
-	if (!checkErr(err, "kernel->setArg(2, *inCLConstBuffer)"))
+	if (!checkErr(err, "kernel->setArg(4, *inCLConstBuffer)"))
 	{
 		emit showErrorMessage(
 			QObject::tr("Cannot set OpenCL argument for %3").arg(QObject::tr("random seed")),
@@ -1065,4 +1177,39 @@ size_t cOpenClEngineRenderFractal::CalcNeededMemory()
 	return max(mem1, mem2);
 }
 
+bool cOpenClEngineRenderFractal::PrepareBufferForBackground(sRenderData *renderData)
+{
+	// buffer for background image
+
+	int texWidth = renderData->textures.backgroundTexture.Width();
+	int texHeight = renderData->textures.backgroundTexture.Height();
+
+	if (backgroungImageBuffer) delete[] backgroungImageBuffer;
+	backgroungImageBuffer = new cl_uchar4[texWidth * texHeight];
+	size_t backgroundImage2DWidth = texWidth;
+	size_t backgroundImage2DHeight = texHeight;
+
+	for (int y = 0; y < texHeight; y++)
+	{
+		for (int x = 0; x < texWidth; x++)
+		{
+			sRGBA16 pixel = renderData->textures.backgroundTexture.FastPixel(x, y);
+			backgroungImageBuffer[x + y * texWidth].s[0] = pixel.R / 256;
+			backgroungImageBuffer[x + y * texWidth].s[1] = pixel.G / 256;
+			backgroungImageBuffer[x + y * texWidth].s[2] = pixel.B / 256;
+			backgroungImageBuffer[x + y * texWidth].s[3] = CL_UCHAR_MAX;
+		}
+	}
+
+	cl_int err;
+	if (backgroundImage2D) delete backgroundImage2D;
+	backgroundImage2D =
+		new cl::Image2D(*hardware->getContext(), CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR,
+			cl::ImageFormat(CL_RGBA, CL_UNORM_INT8), backgroundImage2DWidth, backgroundImage2DHeight,
+			backgroundImage2DWidth * sizeof(cl_uchar4), backgroungImageBuffer, &err);
+	if (!checkErr(err, "cl::Image2D(...backgroundImage...)")) return false;
+
+	return true;
+}
+
 #endif
