@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2016-17 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2016-18 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -118,8 +118,12 @@ void RenderWindow::slotMouseClickOnImage(int x, int y, Qt::MouseButton button) c
 		case RenderedImage::clickPlacePrimitive:
 		case RenderedImage::clickPlaceRandomLightCenter:
 		case RenderedImage::clickGetPoint:
+		case RenderedImage::clickWrapLimitsAroundObject:
 		{
-			gMainInterface->SetByMouse(CVector2<double>(x, y), button, mode);
+			if (gMainInterface->renderedImage->GetEnableClickModes())
+			{
+				gMainInterface->SetByMouse(CVector2<double>(x, y), button, mode);
+			}
 			break;
 		}
 		case RenderedImage::clickDoNothing:
@@ -141,70 +145,174 @@ void RenderWindow::slotChangedComboMouseClickFunction(int index) const
 
 void RenderWindow::slotKeyPressOnImage(QKeyEvent *event)
 {
-
-	int key = event->key();
-	Qt::KeyboardModifiers modifiers = event->modifiers();
-	if (modifiers & Qt::ShiftModifier)
-	{ // Shift pressed
-		switch (key)
-		{
-			case Qt::Key_Up: gMainInterface->RotateCamera("bu_rotate_up"); break;
-			case Qt::Key_Down: gMainInterface->RotateCamera("bu_rotate_down"); break;
-			case Qt::Key_Left: gMainInterface->RotateCamera("bu_rotate_left"); break;
-			case Qt::Key_Right: gMainInterface->RotateCamera("bu_rotate_right"); break;
-			default: break;
-		}
-	}
-	else if (modifiers & Qt::ControlModifier)
-	{ // Ctrl pressed
-		switch (key)
-		{
-			case Qt::Key_Up: gMainInterface->MoveCamera("bu_move_up"); break;
-			case Qt::Key_Down: gMainInterface->MoveCamera("bu_move_down"); break;
-			case Qt::Key_Left: gMainInterface->RotateCamera("bu_rotate_roll_left"); break;
-			case Qt::Key_Right: gMainInterface->RotateCamera("bu_rotate_roll_right"); break;
-			default: break;
-		}
-	}
-	else
-	{
-		// No keyboard modifiers
-		switch (key)
-		{
-			case Qt::Key_W: gMainInterface->MoveCamera("bu_move_up"); break;
-			case Qt::Key_S: gMainInterface->MoveCamera("bu_move_down"); break;
-			case Qt::Key_A: gMainInterface->MoveCamera("bu_move_left"); break;
-			case Qt::Key_D: gMainInterface->MoveCamera("bu_move_right"); break;
-			case Qt::Key_Up: gMainInterface->MoveCamera("bu_move_forward"); break;
-			case Qt::Key_Down: gMainInterface->MoveCamera("bu_move_backward"); break;
-			case Qt::Key_Left: gMainInterface->MoveCamera("bu_move_left"); break;
-			case Qt::Key_Right: gMainInterface->MoveCamera("bu_move_right"); break;
-			default: break;
-		}
-	}
+	currentKeyEvents.append(event->key());
+	lastKeyEventModifiers = event->modifiers();
+	slotKeyHandle();
+	buttonPressTimer->start();
 }
 
 void RenderWindow::slotKeyReleaseOnImage(QKeyEvent *event)
 {
-	(void)event;
+	currentKeyEvents.removeOne(event->key());
+	lastKeyEventModifiers = event->modifiers();
+	slotKeyHandle();
+	buttonPressTimer->start();
 }
 
-void RenderWindow::slotMouseWheelRotatedOnImage(int delta) const
+void RenderWindow::slotButtonLongPress()
 {
-	int index = ui->comboBox_mouse_click_function->currentIndex();
-	QList<QVariant> mode = ui->comboBox_mouse_click_function->itemData(index).toList();
-	RenderedImage::enumClickMode clickMode = RenderedImage::enumClickMode(mode.at(0).toInt());
-	switch (clickMode)
+	slotKeyHandle();
+}
+
+void RenderWindow::slotKeyHandle()
+{
+	if (currentKeyEvents.size() == 0) return;
+
+	gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::read);
+
+	bool render = false;
+
+	for (int i = 0; i < currentKeyEvents.size(); i++)
 	{
-		case RenderedImage::clickPlaceLight:
+		int key = currentKeyEvents.at(i);
+		Qt::KeyboardModifiers modifiers = lastKeyEventModifiers;
+		if (modifiers & Qt::ShiftModifier)
+		{ // Shift pressed
+			switch (key)
+			{
+				case Qt::Key_Up:
+					gMainInterface->MoveCamera("bu_move_forward");
+					render = true;
+					break;
+				case Qt::Key_Down:
+					gMainInterface->MoveCamera("bu_move_backward");
+					render = true;
+					break;
+				case Qt::Key_Left:
+					gMainInterface->MoveCamera("bu_move_left");
+					render = true;
+					break;
+				case Qt::Key_Right:
+					gMainInterface->MoveCamera("bu_move_right");
+					render = true;
+					break;
+				default: break;
+			}
+		}
+		else if (modifiers & Qt::ControlModifier)
+		{ // Ctrl pressed
+			switch (key)
+			{
+				case Qt::Key_Up:
+					gMainInterface->MoveCamera("bu_move_forward");
+					render = true;
+					break;
+				case Qt::Key_Down:
+					gMainInterface->MoveCamera("bu_move_backward");
+					render = true;
+					break;
+				case Qt::Key_Left:
+					gMainInterface->RotateCamera("bu_rotate_roll_left");
+					render = true;
+					break;
+				case Qt::Key_Right:
+					gMainInterface->RotateCamera("bu_rotate_roll_right");
+					render = true;
+					break;
+				default: break;
+			}
+		}
+		else
 		{
-			double deltaLog = exp(delta * 0.001);
-			double dist = ui->widgetEffects->GetAuxLightManualPlacementDistance();
-			dist *= deltaLog;
-			ui->widgetEffects->SetAuxLightManualPlacementDistance(dist);
-			break;
+			// No keyboard modifiers
+			switch (key)
+			{
+				case Qt::Key_W:
+					gMainInterface->MoveCamera("bu_move_up");
+					render = true;
+					break;
+				case Qt::Key_S:
+					gMainInterface->MoveCamera("bu_move_down");
+					render = true;
+					break;
+				case Qt::Key_A:
+					gMainInterface->MoveCamera("bu_move_left");
+					render = true;
+					break;
+				case Qt::Key_D:
+					gMainInterface->MoveCamera("bu_move_right");
+					render = true;
+					break;
+				case Qt::Key_Q:
+					gMainInterface->MoveCamera("bu_move_forward");
+					render = true;
+					break;
+				case Qt::Key_Z:
+					gMainInterface->MoveCamera("bu_move_backward");
+					render = true;
+					break;
+				case Qt::Key_Up:
+					gMainInterface->RotateCamera("bu_rotate_up");
+					render = true;
+					break;
+				case Qt::Key_Down:
+					gMainInterface->RotateCamera("bu_rotate_down");
+					render = true;
+					break;
+				case Qt::Key_Left:
+					gMainInterface->RotateCamera("bu_rotate_left");
+					render = true;
+					break;
+				case Qt::Key_Right:
+					gMainInterface->RotateCamera("bu_rotate_right");
+					render = true;
+					break;
+
+				case Qt::Key_I: gKeyframeAnimation->slotAddKeyframe(); break;
+				case Qt::Key_M:
+					gKeyframeAnimation->slotModifyKeyframe();
+					break;
+				// case Qt::Key_D: gKeyframeAnimation->slotDeleteKeyframe(); break;
+				case Qt::Key_N: gKeyframeAnimation->slotIncreaseCurrentTableIndex(); break;
+				case Qt::Key_P: gKeyframeAnimation->slotDecreaseCurrentTableIndex(); break;
+				default: break;
+			}
+		}
+	}
+	gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::write);
+
+	if (render)
+	{
+		gMainInterface->StartRender();
+	}
+}
+
+void RenderWindow::slotMouseWheelRotatedWithCtrlOnImage(int x, int y, int delta) const
+{
+	if (gMainInterface->renderedImage->GetEnableClickModes())
+	{
+		int index = ui->comboBox_mouse_click_function->currentIndex();
+		QList<QVariant> mode = ui->comboBox_mouse_click_function->itemData(index).toList();
+		RenderedImage::enumClickMode clickMode = RenderedImage::enumClickMode(mode.at(0).toInt());
+		switch (clickMode)
+		{
+			case RenderedImage::clickPlaceLight:
+			{
+				double deltaLog = exp(delta * 0.001);
+				double dist = ui->widgetEffects->GetAuxLightManualPlacementDistance();
+				dist *= deltaLog;
+				ui->widgetEffects->SetAuxLightManualPlacementDistance(dist);
+				break;
+			}
+			case RenderedImage::clickMoveCamera:
+			{
+				Qt::MouseButton button = (delta > 0) ? Qt::LeftButton : Qt::RightButton;
+				mode.append(QVariant(delta));
+				gMainInterface->SetByMouse(CVector2<double>(x, y), button, mode);
+				break;
+			}
+			default: break;
 		}
-		default: break;
 	}
 }
 
@@ -227,6 +335,10 @@ void RenderWindow::slotPopulateToolbar(bool completeRefresh)
 
 	QList<QAction *> actions = ui->toolBar->actions();
 	QStringList toolbarInActions;
+
+	QApplication::connect(
+		mapPresetsFromExamplesLoad, SIGNAL(mapped(QString)), this, SLOT(slotMenuLoadPreset(QString)));
+
 	for (int i = 0; i < actions.size(); i++)
 	{
 		QAction *action = actions.at(i);
@@ -312,8 +424,6 @@ void RenderWindow::slotPopulateToolbar(bool completeRefresh)
 		}
 		QApplication::processEvents();
 	}
-	QApplication::connect(
-		mapPresetsFromExamplesLoad, SIGNAL(mapped(QString)), this, SLOT(slotMenuLoadPreset(QString)));
 
 	QApplication::connect(mapPresetsFromExamplesRemove, SIGNAL(mapped(QString)), this,
 		SLOT(slotMenuRemovePreset(QString)));
