@@ -76,6 +76,7 @@ RenderedImage::RenderedImage(QWidget *parent) : QWidget(parent)
 	anaglyphMode = false;
 	gridType = gridTypeCrosshair;
 	placeLightBehind = false;
+	clickModesEnables = true;
 
 	QList<QVariant> mode;
 	mode.append(int(RenderedImage::clickDoNothing));
@@ -210,7 +211,7 @@ void RenderedImage::DisplayCoordinates()
 		case clickFlightSpeedControl:
 			text = tr("LMB - increase speed");
 			text += tr("\nRMB - decrease speed");
-			text += tr("\narrow keys - strafe");
+			text += tr("\narrow keys - sidewards");
 			text += tr("\nz, x keys - roll");
 			text += tr("\nspacebar - pause");
 			text += tr("\nhold shift key - orthogonal strafe");
@@ -227,6 +228,7 @@ void RenderedImage::DisplayCoordinates()
 			text = tr("Get coordinates");
 			text += tr("\nand distance");
 			break;
+		case clickWrapLimitsAroundObject: text = tr("Wrap limits\naround object"); break;
 	}
 
 	if (clickMode != clickDoNothing)
@@ -579,7 +581,10 @@ void RenderedImage::mousePressEvent(QMouseEvent *event)
 	}
 	else
 	{
-		emit singleClick(event->x(), event->y(), event->button());
+		if (clickModesEnables)
+		{
+			emit singleClick(event->x(), event->y(), event->button());
+		}
 	}
 }
 
@@ -724,19 +729,27 @@ void RenderedImage::keyReleaseEvent(QKeyEvent *event)
 
 void RenderedImage::wheelEvent(QWheelEvent *event)
 {
-	emit mouseWheelRotated(event->delta());
-	if (params)
+	if (clickModesEnables || enumClickMode(clickModeData.at(0).toInt()) == clickFlightSpeedControl)
 	{
-		if (cursorVisible && isFocus && redrawed)
+		if ((event->modifiers() & Qt::ControlModifier) != 0)
 		{
-			update();
+			event->accept(); // do not propagate event to parent widgets - prevents from scrolling
+			emit mouseWheelRotatedWithCtrl(event->x(), event->y(), event->delta());
+			if (params)
+			{
+				if (cursorVisible && isFocus && redrawed)
+				{
+					update();
+				}
+			}
+			else
+			{
+				if (cursorVisible)
+					qCritical()
+						<< "RenderedImage::mouseMoveEvent(QMouseEvent * event): parameters not assigned";
+			}
 		}
 	}
-	else
-	{
-		if (cursorVisible)
-			qCritical() << "RenderedImage::mouseMoveEvent(QMouseEvent * event): parameters not assigned";
-	}
 }
 
 void RenderedImage::DisplayCrosshair() const
